/* GLSL_multilight: different methods to display multiple lights, using plain GL, GLSL shaders or deferred shading
   Copyright (C) 2008 Angelo "Encelo" Theodorou
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <stdio.h>
#include <stdlib.h>

#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glext.h> 

#include "extra.h"

	
int FrameTiming(void)
{
	Uint32 interval;
	static Uint32 current, last = 0, five = 0, nframes = 0;

	current = SDL_GetTicks();
	nframes++;

	if (current - five > 5*1000) {
		printf("%u frames in 5 seconds = %.1f FPS\n", nframes, (float)nframes/5.0f);
		nframes = 0;
		five = current;
	}
	
	interval = current - last;
	last = current;
	
	return interval;
}


void printObjectInfoLog(GLuint obj)
{
	int infologLength = 0;
	int charsWritten  = 0;
	char *infoLog;

	if (glIsShader(obj) == GL_TRUE)
		glGetShaderiv(obj, GL_INFO_LOG_LENGTH, &infologLength);
	else
		glGetProgramiv(obj, GL_INFO_LOG_LENGTH, &infologLength);

	if (infologLength > 1) {
		infoLog = (char *)malloc(infologLength);
		if (glIsShader(obj) == GL_TRUE) {
			glGetShaderInfoLog(obj, infologLength, &charsWritten, infoLog);
			printf("Shader InfoLog: %s\n", infoLog);
		}
		else {
			glGetProgramInfoLog(obj, infologLength, &charsWritten, infoLog);
			printf("Program InfoLog: %s\n", infoLog);
		}
		free(infoLog);
	}
}


void SetLight(Light *light, GLint gl_light)
{
	glLightfv(gl_light, GL_POSITION, light->position);
	glLightfv(gl_light, GL_AMBIENT, light->ambient);
	glLightfv(gl_light, GL_DIFFUSE, light->diffuse);
	glLightfv(gl_light, GL_SPECULAR, light->specular);
}


void DrawLight(Light *light, GLfloat size)
{
	GLfloat x, y, z;

	x = light->position[0];
	y = light->position[1];
	z = light->position[2];

	glColor3f(light->diffuse[0], light->diffuse[1], light->diffuse[2]);
	glPushMatrix();
	glTranslatef(x, y, z);
	glBegin(GL_QUADS);
		glVertex3f((0.5f*size),  (0.5f*size), 0);
		glVertex3f(-(0.5f*size), (0.5f*size), 0);
		glVertex3f(-(0.5f*size),-(0.5f*size), 0);
		glVertex3f((0.5f*size), -(0.5f*size), 0);
	glEnd();
	glPopMatrix();
}


void DrawCube(int open)
{
	glBegin(GL_QUADS);
		if (!open) {
		/* Front Face */
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		}
		/* Back Face */
		if (!open)
			glNormal3f(0.0, 0.0, -1.0);
		else
			glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		if (!open) {
		/* Top Face */
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		}
		/* Bottom Face */
		if (!open)
			glNormal3f(0.0, -1.0, 0.0);
		else
			glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		// Right face
		if (!open)
			glNormal3f(1.0, 0.0, 0.0);
		else
			glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f( 1.0f, -1.0f,  1.0f);
		// Left Face
		if (!open)
			glNormal3f(-1.0, 0.0, 0.0);
		else
			glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(-1.0f,  1.0f, -1.0f);
	glEnd();
}
